/*
	Copyright Adil Qureshi - 1997
	This code is part of gpsys Release 1.1
	and is released for non-commercial use only.
	Questions, comments etc should be forwarded to :-

		Adil Qureshi
		University College London,
		Department of Computer Science,
		Gower St,
		London WC1E 6BT, UK.
		Tel: +44 (0)171 380 7777 x4436
		Fax: +44 (0)171 387 1397
		email: A.Qureshi@cs.ucl.ac.uk
		URL : http://www.cs.ucl.ac.uk/staff/A.Qureshi/
*/

package gpsys;

/**
 * The super class of all GP functions.  A Function is a GP Primitive that
 * takes arguments.  This abstract class should be extended to create
 * your own functions.  One or more of the evaulation methods
 * should be overridden to create the desired functionality.  The reason
 * the evaulation methods have not been declared abstract is to relieve
 * the burden of subclasses having to define all of the different evaluation
 * methods, when only one is needed.
 *
 * @see			gpsys.Primitive
 * @see			gpsys.primitives
 *
 * @version		1.1, 30th June '97
 * @author 	<a href="mailto:A.Qureshi@cs.ucl.ac.uk">Adil Qureshi</a>
 *			<address>Department of Computer Science,</address>
 *			<address>University College London,</address>
 *			<address>Gower St,</address>
 *			<address>London WC1E 6BT,</address>
 *			<address>UK.</address>
 */
public abstract class Function extends Primitive {
	/**
	 * The types of each argument that this function takes.  The length of
	 * this array therefore specifies the number of arguments.
	 */
	public Type argTypes[];

	/**
	 * To be overridden by Functions returning an Object when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	A reference to an Object which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public Object	evaluateObject(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateObject() of abstract class Function invoked");
	}

	/**
	 * To be overridden by Functions returning a byte when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	A byte which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public byte		evaluateByte(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateByte() of abstract class Function invoked");
	}

	/**
	 * To be overridden by Functions returning a short when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	A short which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public short	evaluateShort(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateShort() of abstract class Function invoked");
	}

	/**
	 * To be overridden by Functions returning an int when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	An int which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public int		evaluateInt(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateInt() of abstract class Function invoked");
	}

	/**
	 * To be overridden by Functions returning a long when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	A long which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public long		evaluateLong(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateLong() of abstract class Function invoked");
	}

	/**
	 * To be overridden by Functions returning a float when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	A float which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public float	evaluateFloat(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateFloat() of abstract class Function invoked");
	}

	/**
	 * To be overridden by Functions returning a double when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	A double which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public double	evaluateDouble(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateDouble() of abstract class Function invoked");
	}

	/**
	 * To be overridden by Functions returning a char when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	A char which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public char		evaluateChar(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateChar() of abstract class Function invoked");
	}

	/**
	 * To be overridden by Functions returning a boolean when evaluated.
	 *
	 * @param	i			the individual that is being evaluated.
	 * @param	arguments	the arguments supplied to the Function.
	 * @return	A boolean which is the evaluation result.
	 * @exception	EvaluationException	If this method is called.
	 */
	public boolean	evaluateBoolean(Individual i, Gene[] arguments)
	throws EvaluationException {
		throw new MissingEvaluatorException(
			"evaluateBoolean() of abstract class Function invoked");
	}
}
