/*
	Copyright Adil Qureshi - 1997
	This code is part of gpsys Release 1.1
	and is released for non-commercial use only.
	Questions, comments etc should be forwarded to :-

		Adil Qureshi
		University College London,
		Department of Computer Science,
		Gower St,
		London WC1E 6BT, UK.
		Tel: +44 (0)171 380 7777 x4436
		Fax: +44 (0)171 387 1397
		email: A.Qureshi@cs.ucl.ac.uk
		URL : http://www.cs.ucl.ac.uk/staff/A.Qureshi/
*/

package gpsys;

import java.util.Random;
import java.util.Vector;

/**
 * A GeneBranch defines a Gene subtree of a GP Gene tree.  It has
 * instance variables defining the parent Gene of this subtree, and where it
 * is referenced in the parent.  The GeneBranch provides suffiecient
 * information to allow a subtree to be replaced by another one.  The latter
 * could be generated either by creation (for mutation) or crossover.
 *
 * @version		1.1, 30th June '97
 * @author 	<a href="mailto:A.Qureshi@cs.ucl.ac.uk">Adil Qureshi</a>
 *			<address>Department of Computer Science,</address>
 *			<address>University College London,</address>
 *			<address>Gower St,</address>
 *			<address>London WC1E 6BT,</address>
 *			<address>UK.</address>
 */
public class GeneBranch {
	/**
	 * The parent of the Gene subtree.
	 */
	public Gene parent;

	/**
	 * The arguments index of this Gene subtree in the parent Gene.
	 */
	public int  index;

	/**
	 * The Gene subtree itself.
	 */
	public Gene child;

	/**
	 * The default size of the vector used for keeping subtrees of a given type.
	 */
	public static final int TYPE_VECTOR_SIZE = 10;

	/**
	 * Construct a GeneBranch using the supplied arguments to initialise
	 * the instance variables.
	 *
	 * @param	parent	the parent of this subtree.
	 * @param 	index	the arguments index of this Gene subtree in the
	 *					parent Gene.
	 * @param 	child	the subtree itself.
	 */
	private GeneBranch(Gene parent, int index, Gene child) {
		this.parent = parent;
		this.index  = index;
		this.child  = child;
	}

	/**
	 * select a subtree at random from the given tree.
	 *
	 * @param	rng		the random number generator to be used.
	 * @param	treeTop	the tree from which to select a subtree.
	 */
	public GeneBranch(Random rng, Gene treeTop) {
		parent = null;
		selectBranch(rng, treeTop);
	}

	/**
	 * select a subtree of a given type at random from the given tree.
	 *
	 * @param	rng		the random number generator to be used.
	 * @param	treeTop	the tree from which to select a subtree.
	 * @param	type	the type of the subtree to be selected.
	 */
	public GeneBranch(Random rng, Gene treeTop, Type type) {
		parent = null;
		selectBranch(rng, treeTop, type);
	}

	/**
	 * select a subtree of at random from the given tree.  Initialise the
	 * instance variable to reflect the selected tree.
	 *
	 * @param	rng		the random number generator to be used.
	 * @param	tree	the tree from which to select a subtree.
	 */
	public void selectBranch(Random rng, Gene tree) {
		// until we reach a terminal or our coin tells us to stop
		// descend the tree remembering the parent, and the branch we took.
		while ( !
			((tree instanceof GeneTerminal) || ((rng.nextInt() % 2) == 0) )
			) {
			parent = tree;
			index = rng.nextInt() % ((GeneFunction) tree).arguments.length;
			index = (index < 0) ? -index : index;
			tree = ((GeneFunction) tree).arguments[index];
		}
		child = tree;
		return;
	}

	/**
	 * select a subtree of at random from the given tree.  Initialise the
	 * instance variable to reflect the selected tree.
	 *
	 * @param	rng		the random number generator to be used.
	 * @param	tree	the tree from which to select a subtree.
	 */
	public void selectBranch(Random rng, Gene tree, Type type) {

		// get a table of all the branches of a given type from the tree
		Vector branches = new Vector(TYPE_VECTOR_SIZE, TYPE_VECTOR_SIZE);
		branchesOfType(null, -1, tree, type, branches);

		// if the table is empty, i.e. if there are no branches of this type
		// initialise the instance variables to nulls
		if (branches.size() == 0) {
			parent = child = null;
			index = -1;
			return;
		}

		// select a brach at random from the table of branches
		int branchIndex = rng.nextInt() % branches.size();
		branchIndex = (branchIndex < 0) ? -branchIndex : branchIndex;
		GeneBranch branch = (GeneBranch) branches.elementAt(branchIndex);

		// initialise the instance variables to this branch
		parent	= branch.parent;
		child	= branch.child;
		index	= branch.index;
	}

	/**
	 * Get a table of all subtrees of a given type
	 *
	 * @param	parent		the parent Gene of the child.
	 * @param	index		the arguments index of the  child in the parent.
	 * @param	child		the child Gene subtree.
	 * @param	type		the type of subtree required.
	 * @param	branches	the table of subtrees.
	 */
	// return a vector of all geneBranches of a type starting at child
	public static void branchesOfType(Gene parent, int index, Gene child,
		Type type, Vector branches) {

		// if the child has the same type as required add it to the table
		if (child.p.type == type)
			branches.addElement(new GeneBranch(parent, index, child));

		// if the child is a function, recursive over each argument
		// the parent for these recursively call is the current child
		if (child instanceof GeneFunction)
			for (int i = 0; i < ((GeneFunction) child).arguments.length; i++)
				branchesOfType(child, i, ((GeneFunction) child).arguments[i],
					type, branches);
	}
}
