/*
	Copyright Adil Qureshi - 1997
	This code is part of gpsys Release 1.1
	and is released for non-commercial use only.
	Questions, comments etc should be forwarded to :-

		Adil Qureshi
		University College London,
		Department of Computer Science,
		Gower St,
		London WC1E 6BT, UK.
		Tel: +44 (0)171 380 7777 x4436
		Fax: +44 (0)171 387 1397
		email: A.Qureshi@cs.ucl.ac.uk
		URL : http://www.cs.ucl.ac.uk/staff/A.Qureshi/
*/

package gpsys;

/**
 * A GeneFunctionGrow is a GeneFunction created using the Grow method.
 * In the Grow method, the Gene tree is such that, along any branch, the
 * probability of encountering a Terminal is equal to the probability of
 * encountering a Function.
 *
 * @version		1.1, 30th June '97
 * @author 	<a href="mailto:A.Qureshi@cs.ucl.ac.uk">Adil Qureshi</a>
 *			<address>Department of Computer Science,</address>
 *			<address>University College London,</address>
 *			<address>Gower St,</address>
 *			<address>London WC1E 6BT,</address>
 *			<address>UK.</address>
 */
public class GeneFunctionGrow extends GeneFunction implements Cloneable {
	/**
	 * Create a Gene tree using the Grow method.  The Grow method tries to
	 * creates trees in which along any branch, the probability of
	 * encountering a Terminal is equal to the probability of enountering a
	 * Function.
	 *
	 * @param	maxDepth	the maximum depth of the Gene tree to be generated.
	 * @param	type		the required return type of top node in the tree.
	 * @param	gpParameters	the GP parameters for this run.
	 * @param	adfIndex	the index of the ChromosomeParameters to used.
	 * @exception	TypeException	If a Function or Terminal of a required
	 *								type could not be found.
	 *
	 */
	public GeneFunctionGrow(int maxDepth, Type type,
		GPParameters gpParameters, int adfIndex) throws TypeException {

		// get an array of all functions of the required type
		Function[] functionsOfType =
			gpParameters.adf[adfIndex].functionsOfTypeAtDepthGrow[maxDepth].
				get(type);

		if (functionsOfType == null)
			throw new TypeException(
				"no function of type " + type.toString() +
				" at depth " + maxDepth);

		// then select a function at random from this array and assign to p
		int code =	gpParameters.rng.nextInt() % functionsOfType.length;
		code = (code < 0) ? -code : code;
		Function f = (Function) functionsOfType[code].instance();
		p = f;

		// now generate the arguments to this function
		arguments = new Gene[f.argTypes.length];
		maxDepth--;
		// just for convenience
		TypeToFunctionsTable typeToFunctions =
			gpParameters.adf[adfIndex].functionsOfTypeAtDepthGrow[maxDepth];
		for (int i = 0; i < arguments.length; i++)
			// depth allowing choose a function or a terminal with a 50%
			// chance for either
			if ((maxDepth > 0) &&
				((gpParameters.rng.nextInt() % 2) == 0) &&
				(typeToFunctions.get(f.argTypes[i]) != null)) {
				try {
					arguments[i] = new
						GeneFunctionGrow(maxDepth, f.argTypes[i],
							gpParameters, adfIndex);
				}
				catch(TypeException e) {
					gpParameters.observer.diagnosticUpdate(
						"GeneFunctionGrow " + e.getMessage());
					arguments[i] = new
						GeneTerminal(maxDepth, f.argTypes[i],
							gpParameters, adfIndex);
				}
			} else {
				try {
					arguments[i] = new
						GeneTerminal(maxDepth, f.argTypes[i],
							gpParameters, adfIndex);
				}
				catch(TypeException e) {
					gpParameters.observer.diagnosticUpdate(
						"GeneFunctionGrow " + e.getMessage());
					arguments[i] = new
						GeneFunctionGrow(maxDepth, f.argTypes[i],
							gpParameters, adfIndex);
				}
			}
	}
}
