/*
	Copyright Adil Qureshi - 1997
	This code is part of gpsys Release 1.1
	and is released for non-commercial use only.
	Questions, comments etc should be forwarded to :-

		Adil Qureshi
		University College London,
		Department of Computer Science,
		Gower St,
		London WC1E 6BT, UK.
		Tel: +44 (0)171 380 7777 x4436
		Fax: +44 (0)171 387 1397
		email: A.Qureshi@cs.ucl.ac.uk
		URL : http://www.cs.ucl.ac.uk/staff/A.Qureshi/
*/

package gpsys;

import java.util.Date;

/**
 * A Population holds all the Individuals of a generation.  Additional
 * information is maintained such as the current generation number, the
 * average fitness and complexity of the Individuals, and the best individual
 * in the Population.
 *
 * @version		1.1, 30th June '97
 * @author 	<a href="mailto:A.Qureshi@cs.ucl.ac.uk">Adil Qureshi</a>
 *			<address>Department of Computer Science,</address>
 *			<address>University College London,</address>
 *			<address>Gower St,</address>
 *			<address>London WC1E 6BT,</address>
 *			<address>UK.</address>
 */
public class Population implements java.io.Serializable {
	/**
	 * The Individuals in this population.
	 */
	public Individual[]		p;

	/**
	 * The current generation number.
	 */
	public int				generation;

	/**
	 * The fittest Individual in the Population.
	 */
	public Individual		best;

	/**
	 * The average fitness of the Population.
	 */
	public Fitness			averageFitness;

	/**
	 * The average number of nodes in the Individuals of the Population.
	 */
	public double			averageComplexity;

	/**
	 * The GP parameters for the problem.
	 */
	public GPParameters		gpParameters;

	/*
	 * Create a population of Individuals for the given the problem.
	 *
	 * @param	gpParameters	The parameters for the problem.
	 */
	public Population(GPParameters gpParameters) {
		this.gpParameters = gpParameters;
		generation = 0;
		// create each Individual, telling the observer after each is created
		p = new Individual[gpParameters.populationSize];
		for (gpParameters.creationIndex = 0;
			 gpParameters.creationIndex < gpParameters.populationSize;
			 gpParameters.creationIndex++) {
			p[gpParameters.creationIndex] = new Individual(gpParameters);
			gpParameters.observer.individualUpdate(gpParameters,
				p[gpParameters.creationIndex], gpParameters.creationIndex);
		}
		// now update the statistics for population
		updateStats();
	}

	/*
	 * Evolve the Population by one generation.  The evolver uses a steady
	 * state engine, so that a generation is actually the point at which
	 * an entire population has been created.
	 */
	// steady state evolver
	public void evolve() {
		for (int i = 0; i < p.length; i++) {
			// either perform xover or mutation probabilistically
			if (gpParameters.rng.nextDouble() <= gpParameters.pMutation) {
				// get indices to mum + the one to be replaced by child
				int[] selected = selectForMutation();
				p[selected[1]] = new Individual(gpParameters, p[selected[0]]);
				gpParameters.observer.individualUpdate(gpParameters,
					selected[1], GPObserver.VIA_MUTATION);
			} else {
				// get indices to mum & dad + the one to be replaced by child
				int[] selected = selectForCrossover();

				// mum & dad do crossover (ooooer naughty) to create a child
				p[selected[2]] = new
					Individual(gpParameters, p[selected[0]], p[selected[1]]);
				gpParameters.observer.individualUpdate(gpParameters,
					selected[2], GPObserver.VIA_CROSSOVER);
			}
		}
		updateStats();
		generation++;
	}

	/**
	 * Select Individuals from the Population to be used for mutation.  Two
	 * Individuals are selected, the first is the mother (this is actually
	 * the best Individual from a tournament), and the second (this actually
	 * the worst Individual from a tournament) the Individual to be replaced
	 * by the new child.
	 *
	 * @return	An array of indices to two Individuals, the first is for use
	 *			as the mother and the second is the Individual to be replaced
	 *			by the new child.
	 */
	private int[] selectForMutation() {
		int[] tournament = tournamentSelect();
		int[] selected = new int[2];

		selected[0] = tournament[0];								// mum
		selected[1] = tournament[gpParameters.tournamentSize - 1];	// deceased

		return selected;
	}


	/**
	 * Select Individuals from the Population to be used for crossover.
	 * Three Individuals are selected, the first is the mother (the best
	 * Individual from a tournament), the second is the father (the second best
	 * Individual from a tournament), and the third (the worst Individual from
	 * a tournament) the Individual to be replaced by the new child.
	 *
	 * @return	An array inidices to three Individuals, the first is for use
	 *			as the mother, the second is for use as the father, and the
	 *			third the Individual to be replaced by the child.
	 */
	private int[] selectForCrossover() {
		int[] tournament = tournamentSelect();
		int[] selected = new int[3];

		selected[0] = tournament[0];	// mum
		selected[1] = tournament[1];	// dad
		selected[2] = tournament[gpParameters.tournamentSize - 1]; // deceased

		return selected;
	}

	/**
	 * Perform tournament selection.  N (specified by the tournament size
	 * in the GP parameters) unique individuals are selected at random, and 
	 * sorted in order of Fitness (hightest first).
	 *
	 * @return	An array indices to N Individuals, sorted in order of Fitness
	 */
	private int[] tournamentSelect() {
		int[] tournament = new int[gpParameters.tournamentSize];
		for (int i = 0; i < gpParameters.tournamentSize; i++) {
			// tournament size should be < population size
			boolean notUnique;
			do {
				notUnique = false;
				int tmp = gpParameters.rng.nextInt() % p.length;
				tournament[i] = (tmp < 0) ? -tmp : tmp;
				for (int j = 0; j < i; j++)
					if (tournament[j] == tournament[i]) {
						notUnique = true;
						break;
					}
			} while (notUnique);
		}
		sort(tournament);
		return tournament;
	}


	/**
	 * Sort an array of indices to Individuals so that the first index
	 * references the best Individual, and the last the worst.
	 *
	 * @param	The array of indices to Individuals to be sorted.
	 */
	private void sort(int[] indexList) {
		// used for tournament selection (<= 7 items) hence very simple
		for (int i = 0; i < indexList.length; i++)
			for (int j = i; j < indexList.length; j++)
				if (p[ indexList[i] ].fitness.lessThan(
						p[ indexList[j] ].fitness)) {
					int tmp = indexList[i];
					indexList[i] = indexList[j];
					indexList[j] = tmp;
				}
	}

	/**
	 * Update the statistics for the Population.  This involves finding the
	 * individual with the highest fitness, calculating the average fitness
	 * over the population and calculating the average complexity  over the
	 * population.
	 */
	public void updateStats() {
		// find the individual with the highest fitness
		// calculate the average fitness over the population
		// calculate the average complexity  over the population

		best				= p[0];
		averageComplexity	= best.complexity();
		averageFitness		= best.fitness.instance();
		for (int i = 1; i < p.length; i++) {
			if (best.fitness.lessThan(p[i].fitness))
				best = p[i];
			averageFitness.add(p[i].fitness);
			averageComplexity += p[i].complexity();
		}
		averageFitness.divide(p.length);
		averageComplexity /= p.length;
	}


	/**
	 * Print a report of the population using the specified PrintWriter.  The
	 * report includes the generation number just completed, the date and time
	 * of completion, the average Fitness, the average complexity and a dump
	 * of the best Individual.
	 *
	 * @param The PrintWriter to use to write the report.
	 * 
	 */
	public void report(java.io.PrintWriter pw) {
		pw.println("Generation " + generation + " completed...");
		pw.println("Date :- " +
			java.text.DateFormat.getDateTimeInstance().format(new Date()));
		pw.println("Average fitness " + averageFitness);
		pw.println("Average complexity " + averageComplexity);
		pw.println("best individual = ");;
		pw.println(best);
	}

}
