/*
	Copyright Adil Qureshi - 1997
	This code is part of gpsys Release 1.1
	and is released for non-commercial use only.
	Questions, comments etc should be forwarded to :-

		Adil Qureshi
		University College London,
		Department of Computer Science,
		Gower St,
		London WC1E 6BT, UK.
		Tel: +44 (0)171 380 7777 x4436
		Fax: +44 (0)171 387 1397
		email: A.Qureshi@cs.ucl.ac.uk
		URL : http://www.cs.ucl.ac.uk/staff/A.Qureshi/
*/

package gpsys.primitives;

import gpsys.*;


/**
 * <pre>
 *	BOOLEAN LT(&lt;typeX&gt; lvalue, &lt;typeX&gt; rvalue)
 * </pre>
 *
 * LT is a generic function that tests if the first argument is less than
 * the second argument.
 * LT needs to be Type instantiated during construction to work with particular
 * argument Types.  The supported argument Types include BYTE, SHORT, INT, LONG,
 * FLOAT, DOUBLE, CHAR and BOOLEAN.
 *
 * @see			gpsys.Type
 * @see			gpsys.Primitive
 * @see			gpsys.Function
 *
 * @version     1.1, 30th June '97
 * @author 	<a href="mailto:A.Qureshi@cs.ucl.ac.uk">Adil Qureshi</a>
 *			<address>Department of Computer Science,</address>
 *			<address>University College London,</address>
 *			<address>Gower St,</address>
 *			<address>London WC1E 6BT,</address>
 *			<address>UK.</address>
 */
public class LT extends Function {

	/**
	* Constructs a LT Function that works with the specified Type.  The
	* supported Types include BYTE, SHORT, INT, LONG, FLOAT and DOUBLE.
	*
	* @param type	The Type of this LT Function.
	*/
	public LT(Type argTypes) {
		this.type = Type.BOOLEAN;
		this.argTypes = new Type[2];
		this.argTypes[0] = argTypes;
		this.argTypes[1] = argTypes;
	}

	/**
	* Used to execute the LT test for arguments of any of the supported Types.
	*
	* @param i			The individual being evaluated.
	* @param arguments	The Gene trees representing the arguments to be tested.
	* @return	A boolean whose value is true if the first argument is LT the
	*			second argument, and false otherwise.
	* @exception	EvaluationException	If there is an evaluation failure.
	*/
	public final boolean evaluateBoolean(Individual i, Gene[] arguments)
	throws EvaluationException {
		switch(argTypes[0].typeID) {
			case Type.BYTE_typeID:
				return
					arguments[0].evaluateByte(i) <
					arguments[1].evaluateByte(i);
			case Type.SHORT_typeID:
				return
					arguments[0].evaluateShort(i) <
					arguments[1].evaluateShort(i);
			case Type.INT_typeID:
				return
					arguments[0].evaluateInt(i) <
					arguments[1].evaluateInt(i);
			case Type.LONG_typeID:
				return
					arguments[0].evaluateLong(i) <
					arguments[1].evaluateLong(i);
			case Type.FLOAT_typeID:
				return
					arguments[0].evaluateFloat(i) <
					arguments[1].evaluateFloat(i);
			case Type.DOUBLE_typeID:
				return
					arguments[0].evaluateDouble(i) <
					arguments[1].evaluateDouble(i);
			default:
				System.out.println("LT - type " + type + "unknown");
				return false;
		}
	}

	/**
	* Creates an instance of this LT object.  The Type instantiation is
	* preserved so that if an INT LT is being cloned, the clone will also be
	* INT instantiated.
	*
	* @return	A reference to this Object (yes the same object, since
	*			the type information is to be the same, there are therefore
	*			no instance variables to be modified.
	*/
	public final Primitive instance() {
		return this;
	}

	/**
	* Returns a String representation of this Function.
	*
	* @return A String containing the name of the Function.
	*/
	public String toString() {
		return "LT";
	}
}
